import os
import abacusSoftware.constants as constants
import pyAbacus as abacus
from PyQt5 import QtGui

def timeInUnitsToMs(time):
    value = 0
    if 'ms' in time:
        value = int(time.replace('ms', ''))
    elif 's' in time:
        value = int(time.replace('s', ''))*1000
    return value

def setSamplingComboBox(comboBox, value = abacus.DEFAULT_SAMP):
    comboBox.clear()

    model = comboBox.model()
    for row in abacus.SAMP_VALUES:
        item = QtGui.QStandardItem(row)
        if timeInUnitsToMs(row) < abacus.SAMP_CUTOFF:
            item.setBackground(QtGui.QColor('red'))
            item.setForeground(QtGui.QColor('white'))
        model.appendRow(item)

    comboBox.setCurrentIndex(comboBox.findText(value))

def setCoincidenceSpinBox(spinBox, value = abacus.DEFAULT_COIN):
    spinBox.setMinimum(abacus.MIN_COIN)
    spinBox.setMaximum(abacus.MAX_COIN)
    spinBox.setSingleStep(abacus.STEP_COIN)
    spinBox.setValue(value)

def setDelaySpinBox(spinBox, value = abacus.DEFAULT_DELAY):
    spinBox.setMinimum(abacus.MIN_DELAY)
    spinBox.setMaximum(abacus.MAX_DELAY)
    spinBox.setSingleStep(abacus.STEP_DELAY)
    spinBox.setValue(value)

def setSleepSpinBox(spinBox, value = abacus.DEFAULT_SLEEP):
    spinBox.setMinimum(abacus.MIN_SLEEP)
    spinBox.setMaximum(abacus.MAX_SLEEP)
    spinBox.setSingleStep(abacus.STEP_SLEEP)
    spinBox.setValue(value)

def findWidgets(class_, widget):
    return [att for att in dir(class_) if widget in att]

def unicodePath(path):
    return path.replace("\\", "/")

def readConstantsFile():
    if os.path.exists(constants.SETTINGS_PATH):
        with open(constants.SETTINGS_PATH) as file:
            for line in file:
                try:
                    exec("constants.%s"%line)
                except SyntaxError as e:
                    pass
        constants.SETTING_FILE_EXISTS = True
    else:
        print("Settings file not found at: %s"%constants.SETTINGS_PATH)

def updateConstants(class_):
    for (name, action) in zip(constants.WIDGETS_NAMES, constants.WIDGETS_SET_ACTIONS):
        attributes = findWidgets(class_, name)
        for att in attributes:
            if att in dir(constants):
                val = eval("constants.%s"%att)
                if name != "comboBox":
                    exec(action%(att, val))
                else:
                    exec(action%(att, att, val))

def findDocuments():
    if constants.CURRENT_OS == "win32":
        import ctypes.wintypes
        buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
        ctypes.windll.shell32.SHGetFolderPathW(None, 5, None, 0, buf)
        buf = buf.value
    else:
        buf = os.path.expanduser("~")
    return buf



import os
import numpy as np
from PyQt5 import QtWidgets, QtGui, QtCore
from PyQt5.QtGui import QTableWidgetItem
from pyAbacus.constants import CURRENT_OS

import abacusSoftware.common as common
import abacusSoftware.constants as constants
from pyAbacus.communication import findPorts

class Table(QtWidgets.QTableWidget):
    def __init__(self, cols):
        QtWidgets.QTableWidget.__init__(self)
        self.setColumnCount(cols)
        self.horizontalHeader().setSortIndicatorShown(False)
        self.verticalHeader().setDefaultSectionSize(18)
        self.verticalHeader().setMinimumSectionSize(18)
        self.verticalHeader().setSortIndicatorShown(False)

        self.last_time = None

        self.headers = ['time (s)', 'A', 'B', 'AB']
        self.setHorizontalHeaderLabels(self.headers)
        self.resizeRowsToContents()
        self.resizeColumnsToContents()

        self.horizontalHeader().setResizeMode(QtWidgets.QHeaderView.Stretch)
        self.horizontalHeader().setStretchLastSection(True);

    def insertData(self, data):
        rows, cols = data.shape

        if self.last_time == None:
            self.last_time = data[0, 0]
            index = 0
        else:
            index = np.where(data[:, 0] == self.last_time)[0][0]
            self.last_time = data[-1, 0]
            data = data[index + 1:]

        for i in range(data.shape[0]):
            self.insertRow(0)
            for j in range(cols):
                if j == 0:
                    fmt = "%.3f"
                else:
                    fmt = "%d"
                self.setItem(0, j, QTableWidgetItem(fmt%data[i, j]))
                self.item(0, j).setTextAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignVCenter)

class AutoSizeLabel(QtWidgets.QLabel):
    """ From reclosedev at http://stackoverflow.com/questions/8796380/automatically-resizing-label-text-in-qt-strange-behaviour
    and Jean-Sébastien http://stackoverflow.com/questions/29852498/syncing-label-fontsize-with-layout-in-pyqt
    """
    MAX_DIGITS = 7 #: Maximum number of digits of a number in label.
    MAX_CHARS = 9 + MAX_DIGITS #: Maximum number of letters in a label.
    global CURRENT_OS
    def __init__(self, text, value):
        QtWidgets.QLabel.__init__(self)
        self.setSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        self.font_name = "Monospace"
        if CURRENT_OS == "win32":
            self.font_name = "Courier New"
        self.setFont(QtGui.QFont(self.font_name))
        self.initial_font_size = 10
        self.font_size = 10
        self.MAX_TRY = 150
        self.height = self.contentsRect().height()
        self.width = self.contentsRect().width()
        self.name = text
        self.value = value
        self.setText(self.stylishText(text, value))
        self.setFontSize(self.font_size)

    def setFontSize(self, size):
        """ Changes the size of the font to `size` """
        f = self.font()
        f.setPixelSize(size)
        self.setFont(f)

    def setColor(self, color):
        """ Sets the font color.
        Args:
            color (string): six digit hexadecimal color representation.
        """
        self.setStyleSheet('color: %s'%color)

    def stylishText(self, text, value):
        """ Uses and incomning `text` and `value` to create and text of length
        `MAX_CHARS`, filled with spaces.
        Returns:
            string: text of length `MAX_CHARS`.
        """
        n_text = len(text)
        n_value = len(value)
        N = n_text + n_value
        spaces = [" " for i in range(self.MAX_CHARS - N-1)]
        spaces = "".join(spaces)
        text = "%s: %s%s"%(text, spaces, value)
        return text

    def changeValue(self, value):
        """ Sets the text in label with its name and its value. """
        if type(value) is not str:
            value = "%d"%value
        if self.value != value:
            self.value = value
            self.setText(self.stylishText(self.name, self.value))

    def resize(self):
        """ Finds the best font size to use if the size of the window changes. """
        f = self.font()
        cr = self.contentsRect()
        height = cr.height()
        width = cr.width()
        if abs(height*width - self.height*self.width) > 1:
            self.font_size = self.initial_font_size
            for i in range(self.MAX_TRY):
                f.setPixelSize(self.font_size)
                br =  QtGui.QFontMetrics(f).boundingRect(self.text())
                if br.height() <= cr.height() and br.width() <= cr.width():
                    self.font_size += 1
                else:
                    if CURRENT_OS == 'win32':
                        self.font_size += -1

                    else:
                        self.font_size += -2
                    f.setPixelSize(max(self.font_size, 1))
                    break
            self.setFont(f)
            self.height = height
            self.width = width

class CurrentLabels(QtWidgets.QWidget):
    def __init__(self, parent = None):
        QtWidgets.QWidget.__init__(self, parent)
        # self.setSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        self.layout = QtWidgets.QVBoxLayout(parent)
        self.installEventFilter(self)
        self.labels = []

    def createLabels(self, labels=["counts A", "counts B", "coinc AB"]):
        for name in labels:
            label = AutoSizeLabel(name, "0")
            self.layout.addWidget(label)
            self.labels.append(label)

    # def createLabels(self, detectors, coincidences):
    #     for detector in detectors:
    #         name = detector.name
    #         label = AutoSizeLabel(name, "0")
    #         self.layout.addWidget(label)
    #         self.labels.append(label)
    #
    #     for coin in coincidences:
    #         name = coin.name
    #         label = AutoSizeLabel(name, "0")
    #         self.layout.addWidget(label)
    #         self.labels.append(label)

    def setColor(self, label, color):
        label.setColor(color)

    def setColors(self, colors):
        for (label, color) in zip(self.labels, colors):
            self.setColor(label, color)

    def changeValue(self, index, value):
        self.labels[index].changeValue(value)

    def eventFilter(self, object, evt):
        """ Checks if there is the window size has changed.
        Returns:
            boolean: True if it has not changed. False otherwise. """
        ty = evt.type()
        if ty == 97: # DONT KNOW WHY
            self.resizeEvent(evt)
            return False
        elif ty == 12:
            self.resizeEvent(evt)
            return False
        else:
            return True

    def resizeEvent(self, evt):
        sizes = [None]*3
        try:
            for (i, label) in enumerate(self.labels):
                label.resize()
                sizes[i] = label.font_size

            if len(self.labels) > 0:
                size = max(sizes)
                for label in self.labels:
                    label.setFontSize(size)
        except Exception as e:
            print(e)

class ConnectDialog(QtWidgets.QDialog):
    def __init__(self):
        QtWidgets.QDialog.__init__(self)
        self.verticalLayout = QtWidgets.QVBoxLayout(self)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setSpacing(6)

        self.frame = QtWidgets.QFrame()

        self.horizontalLayout = QtWidgets.QHBoxLayout(self.frame)
        self.horizontalLayout.setContentsMargins(11, 11, 11, 11)
        self.horizontalLayout.setSpacing(6)

        self.label = QtWidgets.QLabel()

        self.verticalLayout.addWidget(self.label)
        self.verticalLayout.addWidget(self.frame)

        self.comboBox = QtWidgets.QComboBox()
        self.comboBox.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)

        self.refresh_button = QtWidgets.QPushButton()
        self.refresh_button.setText("Refresh")
        self.refresh_button.clicked.connect(self.refresh)

        self.horizontalLayout.addWidget(self.comboBox)
        self.horizontalLayout.addWidget(self.refresh_button)

        self.label.setText(constants.CONNECT_LABEL)
        self.label.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        self.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)

        self.setWindowTitle("Tausand Abacus device selection")
        self.setMinimumSize(QtCore.QSize(450, 100))

        self.buttons = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel,
            QtCore.Qt.Horizontal, self)

        self.verticalLayout.addWidget(self.buttons)
        self.buttons.accepted.connect(self.accept)
        self.buttons.rejected.connect(self.reject2)

        self.ports = None

    def refresh(self):
        self.clear()
        self.ports = findPorts()
        ports_names = list(self.ports.keys())
        if len(ports_names) == 0:
            self.label.setText(constants.CONNECT_EMPTY_LABEL)
        else:
            self.label.setText(constants.CONNECT_LABEL)
        self.comboBox.addItems(ports_names)
        self.adjustSize()

    def clear(self):
        self.comboBox.clear()

    def reject2(self):
        self.clear()
        self.reject()

class SettingsDialog(QtWidgets.QDialog):
    def __init__(self, parent):
        QtWidgets.QDialog.__init__(self)
        # self.setWindowFlags(QtCore.Qt.WindowSystemMenuHint  | QtCore.Qt.WindowTitleHint)

        self.parent = parent
        self.setWindowTitle("Default settings")

        self.verticalLayout = QtWidgets.QVBoxLayout(self)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setSpacing(6)

        self.tabWidget = QtWidgets.QTabWidget(self)

        self.file_tab = QtWidgets.QWidget()
        self.settings_tab = QtWidgets.QWidget()

        self.tabWidget.addTab(self.file_tab, "File")
        self.tabWidget.addTab(self.settings_tab, "Settings")

        self.verticalLayout.addWidget(self.tabWidget)

        """
        file tab
        """
        self.file_tab_verticalLayout = QtWidgets.QVBoxLayout(self.file_tab)

        # frame1
        self.file_tab_frame1 = QtWidgets.QFrame()
        self.file_tab_frame1_layout = QtWidgets.QHBoxLayout(self.file_tab_frame1)

        self.directory_label = QtWidgets.QLabel("Directory:")
        self.directory_lineEdit = ClickableLineEdit()
        self.directory_pushButton = QtWidgets.QPushButton("Open")

        self.file_tab_frame1_layout.addWidget(self.directory_label)
        self.file_tab_frame1_layout.addWidget(self.directory_lineEdit)
        self.file_tab_frame1_layout.addWidget(self.directory_pushButton)

        self.file_tab_verticalLayout.addWidget(self.file_tab_frame1)
        self.directory_lineEdit.clicked.connect(self.chooseFolder)
        self.directory_pushButton.clicked.connect(self.chooseFolder)

        # frame2
        self.file_tab_frame2 = QtWidgets.QFrame()
        self.file_tab_frame2_layout = QtWidgets.QFormLayout(self.file_tab_frame2)

        self.file_prefix_label = QtWidgets.QLabel("File prefix:")
        self.file_prefix_lineEdit = QtWidgets.QLineEdit()
        self.extension_label = QtWidgets.QLabel("Extension:")
        self.extension_comboBox = QtWidgets.QComboBox()
        self.delimiter_label = QtWidgets.QLabel("Delimiter:")
        self.delimiter_comboBox = QtWidgets.QComboBox()
        self.parameters_label = QtWidgets.QLabel("Parameters suffix:")
        self.parameters_lineEdit = QtWidgets.QLineEdit()
        self.autogenerate_label = QtWidgets.QLabel("Autogenerate file name:")
        self.autogenerate_checkBox = QtWidgets.QCheckBox()
        self.check_updates_label = QtWidgets.QLabel("Check for updates:")
        self.check_updates_checkBox = QtWidgets.QCheckBox()
        self.datetime_label = QtWidgets.QLabel("Use datetime:")
        self.datetime_checkBox = QtWidgets.QCheckBox()

        self.file_tab_verticalLayout.addWidget(self.file_tab_frame2)

        widgets = [(self.check_updates_label, self.check_updates_checkBox),
                    (self.autogenerate_label, self.autogenerate_checkBox),
                    (self.datetime_label, self.datetime_checkBox),
                    (self.file_prefix_label, self.file_prefix_lineEdit),
                    (self.parameters_label, self.parameters_lineEdit),
                    (self.extension_label, self.extension_comboBox),
                    (self.delimiter_label, self.delimiter_comboBox),
                    ]

        self.fillFormLayout(self.file_tab_frame2_layout, widgets)

        self.file_tab_verticalLayout.addWidget(self.file_tab_frame2)

        self.autogenerate_checkBox.setCheckState(2)
        self.check_updates_checkBox.setCheckState(2)
        self.autogenerate_checkBox.stateChanged.connect(self.actogenerateMethod)
        self.datetime_checkBox.setCheckState(2)
        self.parameters_lineEdit.setText(constants.PARAMS_SUFFIX)
        self.file_prefix_lineEdit.setText(constants.FILE_PREFIX)
        self.directory_lineEdit.setText(common.findDocuments())
        self.delimiter_comboBox.insertItems(0, constants.DELIMITERS)
        self.extension_comboBox.insertItems(0, sorted(constants.SUPPORTED_EXTENSIONS.keys())[::-1])

        """
        settings tab
        """
        self.settings_tab_verticalLayout = QtWidgets.QVBoxLayout(self.settings_tab)

        self.settings_tab_frame = QtWidgets.QFrame()
        self.settings_tab_frame_layout = QtWidgets.QFormLayout(self.settings_tab_frame)

        self.sampling_label = QtWidgets.QLabel("Sampling time:")
        self.sampling_comboBox = QtWidgets.QComboBox()
        self.coincidence_label = QtWidgets.QLabel("Coincidence window (ns):")
        self.coincidence_spinBox = QtWidgets.QSpinBox()
        self.delayA_label = QtWidgets.QLabel("Delay A (ns):")
        self.delayA_spinBox = QtWidgets.QSpinBox()
        self.delayB_label = QtWidgets.QLabel("Delay B (ns):")
        self.delayB_spinBox = QtWidgets.QSpinBox()
        self.sleepA_label = QtWidgets.QLabel("Sleep time A (ns):")
        self.sleepA_spinBox = QtWidgets.QSpinBox()
        self.sleepB_label = QtWidgets.QLabel("Sleep time B (ns):")
        self.sleepB_spinBox = QtWidgets.QSpinBox()

        self.sampling_comboBox.setEditable(True)
        self.sampling_comboBox.lineEdit().setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.sampling_comboBox.lineEdit().setReadOnly(True)
        self.coincidence_spinBox.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.delayA_spinBox.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.delayB_spinBox.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.sleepA_spinBox.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.sleepB_spinBox.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        widgets = [(self.sampling_label, self.sampling_comboBox),
                    (self.coincidence_label, self.coincidence_spinBox),
                    (self.delayA_label, self.delayA_spinBox),
                    (self.delayB_label, self.delayB_spinBox),
                    (self.sleepA_label, self.sleepA_spinBox),
                    (self.sleepB_label, self.sleepB_spinBox),]
                    # (self.from_device_label, self.from_device_checkBox)]

        self.fillFormLayout(self.settings_tab_frame_layout, widgets)

        self.settings_tab_verticalLayout.addWidget(self.settings_tab_frame)

        common.setSamplingComboBox(self.sampling_comboBox)
        common.setCoincidenceSpinBox(self.coincidence_spinBox)
        common.setDelaySpinBox(self.delayA_spinBox)
        common.setDelaySpinBox(self.delayB_spinBox)
        common.setSleepSpinBox(self.sleepA_spinBox)
        common.setSleepSpinBox(self.sleepB_spinBox)

        """
        buttons
        """
        self.buttons = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel,
            QtCore.Qt.Horizontal, self)

        self.buttons.button(QtWidgets.QDialogButtonBox.Ok).setText("Apply")
        self.buttons.accepted.connect(self.accept_replace)
        self.buttons.rejected.connect(self.reject)

        self.verticalLayout.addWidget(self.buttons)

        self.setConstants()
        self.constantsWriter()

    def actogenerateMethod(self, val):
        self.datetime_checkBox.setEnabled(val)
        self.file_prefix_lineEdit.setEnabled(val)

    def fillFormLayout(self, layout, values):
        for (i, line) in enumerate(values):
            layout.setWidget(i, QtWidgets.QFormLayout.LabelRole, line[0])
            layout.setWidget(i, QtWidgets.QFormLayout.FieldRole, line[1])

    def constantsWriter(self):
        lines = []
        for (widget, eval_) in zip(constants.WIDGETS_NAMES, constants.WIDGETS_GET_ACTIONS):
            complete = common.findWidgets(self, widget)
            for item in complete:
                val = eval(eval_%item)
                if type(val) is str:
                    if item == "directory_lineEdit":
                        val = common.unicodePath(val)
                    string = "%s = '%s'"%(item, val)
                else:
                    string = "%s = %s"%(item, val)
                lines.append(string)
        self.writeDefault(lines)
        lines += ["EXTENSION_DATA = '%s'"%self.extension_comboBox.currentText(),
                    "EXTENSION_PARAMS = '%s.txt'"%self.parameters_lineEdit.text()]
        delimiter = self.delimiter_comboBox.currentText()
        if delimiter == "Tab":
            delimiter = "\t"
        elif delimiter == "Space":
            delimiter = " "
        lines += ["DELIMITER = '%s'"%delimiter]
        self.updateConstants(lines)
        self.parent.updateConstants()

    def accept_replace(self):
        self.constantsWriter()
        self.accept()

    def writeDefault(self, lines):
        try:
            with open(constants.SETTINGS_PATH, "w") as file:
                [file.write(line + constants.BREAKLINE) for line in lines]
        except FileNotFoundError as e:
            print(e)

    def updateConstants(self, lines):
        [exec("constants.%s"%line) for line in lines]

    def setConstants(self):
        try:
            common.updateConstants(self)
        except AttributeError:
            pass

    def chooseFolder(self):
        folder = str(QtWidgets.QFileDialog.getExistingDirectory(self, "Select Directory", common.findDocuments()))
        if folder != "":
            self.directory_lineEdit.setText(folder)

class SubWindow(QtWidgets.QMdiSubWindow):
    def __init__(self, parent = None):
        super(SubWindow, self).__init__(parent)
        self.parent = parent

    def closeEvent(self, evnt):
        evnt.ignore()
        self.hide()
        name = self.windowTitle().lower()
        actions = self.parent.menuView.actions()
        for action in actions:
            if name in action.text(): action.setChecked(False)

class ClickableLineEdit(QtGui.QLineEdit):
    clicked = QtCore.pyqtSignal()
    def __init__(self, parent = None):
        super(ClickableLineEdit, self).__init__(parent)
        self.setReadOnly(True)

    def mousePressEvent(self, event):
        self.clicked.emit()
        QtGui.QLineEdit.mousePressEvent(self, event)



import abacusSoftware.constants as constants
import urllib.request

URL_VERSION = "https://raw.githubusercontent.com/Tausand-dev/AbacusSoftware/master/Software/constants.py"
TARGET_URL = "https://sourceforge.net/projects/quantum-temp/"

def versionstr(version):
    if "=" in version:
        version = version.split("=")[-1].replace('"', "").replace("'", "")
    version = version.split(".")
    return [int(v) for v in version]

def checkUpdate():
    try:
        with urllib.request.urlopen(URL_VERSION) as response:
           html = response.read().decode().split("\n")
           for line in html:
               if "__version__" in line:
                   url_version = versionstr(line)
                   break
    except Exception as e:
        url_version = 0

    current_version = versionstr(constants.__version__)
    n = min(len(url_version), len(current_version))

    for i in range(n):
        if url_version[i] > current_version[i]:
            return ".".join([str(v) for v in url_version])

    return None



import abacusSoftware.constants as constants
import abacusSoftware.common as common
from abacusSoftware.files import File
import pyAbacus as abacus


import numpy as np
import pyqtgraph as pg
from threading import Thread
from PyQt5 import QtWidgets, QtGui, QtCore
from abacusSoftware.supportWidgets import ClickableLineEdit

class SweepDialogBase(QtWidgets.QDialog):
    def __init__(self, parent):
        super(SweepDialogBase, self).__init__(parent)
        self.resize(400, 500)

        self.parent = parent

        self.verticalLayout = QtWidgets.QVBoxLayout(self)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setSpacing(6)

        self.frame = QtWidgets.QFrame()

        self.horizontalLayout = QtWidgets.QHBoxLayout(self.frame)
        self.horizontalLayout.setContentsMargins(11, 11, 11, 11)
        self.horizontalLayout.setSpacing(6)

        label = QtWidgets.QLabel("Save as:")
        self.lineEdit = ClickableLineEdit(self)
        self.lineEdit.clicked.connect(self.chooseFile)

        self.horizontalLayout.addWidget(label)
        self.horizontalLayout.addWidget(self.lineEdit)
        self.verticalLayout.addWidget(self.frame)

        self.groupBox = QtWidgets.QGroupBox("Settings")

        self.formLayout = QtWidgets.QFormLayout(self.groupBox)

        samplingLabel = QtWidgets.QLabel("Sampling time:")

        startLabel = QtWidgets.QLabel("Start time (ns):")
        stopLabel = QtWidgets.QLabel("Stop time (ns):")
        stepLabel = QtWidgets.QLabel("Step size (ns):")
        nLabel = QtWidgets.QLabel("Number of measurements per step:")

        self.samplingLabel = QtWidgets.QLabel(self.parent.sampling_comboBox.currentText())
        self.startSpin = QtWidgets.QSpinBox()
        self.stopSpin = QtWidgets.QSpinBox()
        self.stepSpin = QtWidgets.QSpinBox()
        self.nSpin = QtWidgets.QSpinBox()
        self.nSpin.setMinimum(1)

        self.samplingLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.startSpin.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.stopSpin.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.stepSpin.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.nSpin.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        self.startSpin.valueChanged.connect(self.handleStart)

        self.formLayout.addRow(samplingLabel, self.samplingLabel)
        self.formLayout.addRow(startLabel, self.startSpin)
        self.formLayout.addRow(stopLabel, self.stopSpin)
        self.formLayout.addRow(stepLabel, self.stepSpin)
        self.formLayout.addRow(nLabel, self.nSpin)

        self.verticalLayout.addWidget(self.groupBox)

        self.startStopButton = QtWidgets.QPushButton("Start")
        self.startStopButton.setMaximumSize(QtCore.QSize(140, 60))
        self.verticalLayout.addWidget(self.startStopButton, alignment = QtCore.Qt.AlignRight)

        self.plot_win = pg.GraphicsWindow()
        self.plot = self.plot_win.addPlot()

        symbolSize = 5
        self.plot_line = self.plot.plot(pen = "r", symbol='o', symbolPen = "r", symbolBrush="r", symbolSize=symbolSize)
        self.verticalLayout.addWidget(self.plot_win)

        self.fileName = ""

        self.startStopButton.clicked.connect(self.startStop)

        self.x_data = []
        self.y_data = []

        self.completed = False

        self.timer = QtCore.QTimer()
        self.timer.setInterval(constants.CHECK_RATE)
        self.timer.timeout.connect(self.updatePlot)

        self.header = None

        self.error = None

    def handleStart(self, value):
        self.stopSpin.setMinimum(value + abacus.STEP_DELAY)

    def warning(self, error):
        error_text = str(error)
        msg = QtWidgets.QMessageBox()
        msg.setIcon(QtWidgets.QMessageBox.Warning)
        msg.setText(error_text)
        msg.setWindowTitle("Warning")
        msg.setStandardButtons(QtWidgets.QMessageBox.Ok | QtWidgets.QMessageBox.Cancel)
        return msg.exec_()

    def enableWidgets(self, enable):
        self.startSpin.setEnabled(enable)
        self.stopSpin.setEnabled(enable)
        self.stepSpin.setEnabled(enable)
        self.nSpin.setEnabled(enable)
        try:
            self.comboBox.setEnabled(enable)
        except:
            pass

    def updatePlot(self):
        self.plot_line.setData(self.x_data, self.y_data)
        if self.error != None:
            self.parent.errorWindow(self.error)
            self.error = None

        if self.completed:
            if self.fileName != "":
                file = File(self.fileName, self.header)
                data = np.vstack((self.x_data, self.y_data)).T
                file.npwrite(data, "%d" + constants.DELIMITER + "%d")

            self.x_data = []
            self.y_data = []
            self.timer.stop()
            self.completed = False
            self.startStopButton.setText("Start")
            self.startStopButton.setStyleSheet("background-color: none")
            self.enableWidgets(True)
            self.parent.check_timer.start()

    def cleanPlot(self):
        self.x_data = []
        self.y_data = []
        self.plot_line.setData(self.x_data, self.y_data)

    def chooseFile(self):
        try:
            directory = constants.directory_lineEdit
        except:
            directory = os.path.expanduser("~")

        dlg = QtWidgets.QFileDialog(directory = directory)
        dlg.setAcceptMode(QtWidgets.QFileDialog.AcceptSave)
        dlg.setFileMode(QtWidgets.QFileDialog.AnyFile)
        nameFilters = [constants.SUPPORTED_EXTENSIONS[extension] for extension in constants.SUPPORTED_EXTENSIONS]
        dlg.setNameFilters(nameFilters)
        dlg.selectNameFilter(constants.SUPPORTED_EXTENSIONS[constants.EXTENSION_DATA])
        if dlg.exec_():
            name = dlg.selectedFiles()[0]
            self.fileName = common.unicodePath(name)
            self.lineEdit.setText(self.fileName)

    def stopAcquisition(self):
        e = Exception("Data acquisition is active, in order to make the sweep it will be turned off.")
        ans = self.warning(e)
        if ans == QtWidgets.QMessageBox.Ok:
            ans = True
        else: ans = False
        if ans: self.parent.startAcquisition()
        return ans

class DelayDialog(SweepDialogBase):
    def __init__(self, parent):
        super(DelayDialog, self).__init__(parent)

        self.setWindowTitle("Delay time sweep")

        self.startSpin.setMinimum(-abacus.MAX_DELAY)
        self.startSpin.setMaximum(abacus.MAX_DELAY - abacus.STEP_DELAY)
        self.startSpin.setSingleStep(abacus.STEP_DELAY)
        self.startSpin.setValue(-abacus.MAX_DELAY)

        self.stopSpin.setMinimum(-abacus.MAX_DELAY)
        self.stopSpin.setMaximum(abacus.MAX_DELAY)
        self.stopSpin.setSingleStep(abacus.STEP_DELAY)
        self.stopSpin.setValue(abacus.MAX_DELAY)

        self.stepSpin.setMinimum(abacus.STEP_DELAY)
        self.stepSpin.setMaximum(((abacus.MAX_DELAY - abacus.MIN_DELAY) // abacus.STEP_DELAY) * abacus.STEP_DELAY)
        self.stepSpin.setSingleStep(abacus.STEP_DELAY)

        self.plot.setLabel('left', "Coincidences")
        self.plot.setLabel('bottom', "Delay time", units='ns')

    def startStop(self):
        if self.startStopButton.text() == "Stop":
            self.timer.stop()
            self.completed = True
            self.updatePlot()
            self.completed = True

        else:
            step = self.stepSpin.value()
            n = self.nSpin.value()
            range_ = np.arange(self.startSpin.value(), self.stopSpin.value() + step, step)
            range_ = range_[range_ <= abacus.MAX_DELAY]

            if self.parent.experiment != None:
                if self.parent.streaming:
                    if self.stopAcquisition():
                        self.run(n, range_)
                else:
                    self.run(n, range_)
            else:
                self.parent.connect()
                if self.parent.experiment != None:
                    if self.parent.streaming:
                        if self.stopAcquisition():
                            self.run(n, range_)
                    else:
                        self.run(n, range_)

    def run(self, n, range_):
        self.cleanPlot()
        self.completed = False
        self.startStopButton.setText("Stop")
        self.startStopButton.setStyleSheet("background-color: green")
        self.enableWidgets(False)

        self.header = "Delay time (ns)"  + constants.DELIMITER +  "Coincidences"

        self.parent.check_timer.stop()
        thread = Thread(target = self.heavyDuty, args = (n, range_))
        thread.daemon = True
        self.timer.start()
        thread.start()

    def heavyDuty(self, n, range_):
        try:
            for (i, delay) in enumerate(range_):
                if not self.completed:
                    if delay < 0:
                        delay1 = abs(delay)
                        delay2 = 0
                    else:
                        delay1 = 0
                        delay2 = delay

                    result = self.parent.experiment.delaySweep("A", "B", delay1, delay2, n)
                    values = self.parent.experiment.delaySweep("A", "B", delay1, delay2, n)
                    result = []
                    for value in values:
                        result.append(value)

                    self.x_data.append(delay)
                    self.y_data.append(np.mean(result))
                else:
                    break

            self.completed = True
        except Exception as e:
            self.completed = True
            self.error = e

class SleepDialog(SweepDialogBase):
    def __init__(self, parent):
        super(SleepDialog, self).__init__(parent)

        self.parent = parent

        self.setWindowTitle("Sleep time sweep")

        label = QtWidgets.QLabel("Channel:")
        self.comboBox = QtWidgets.QComboBox()
        self.comboBox.addItems(["A", "B"])
        self.comboBox.setEditable(True)
        self.comboBox.lineEdit().setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.comboBox.lineEdit().setReadOnly(True)

        self.formLayout.insertRow(0, label, self.comboBox)

        self.startSpin.setMinimum(abacus.MIN_SLEEP)
        self.startSpin.setMaximum(abacus.MAX_SLEEP - abacus.STEP_SLEEP)
        self.startSpin.setSingleStep(abacus.STEP_SLEEP)
        self.startSpin.setValue(abacus.MIN_SLEEP)

        self.stopSpin.setMinimum(abacus.MIN_SLEEP)
        self.stopSpin.setMaximum(abacus.MAX_SLEEP)
        self.stopSpin.setSingleStep(abacus.STEP_SLEEP)
        self.stopSpin.setValue(abacus.MAX_SLEEP)

        self.stepSpin.setMinimum(abacus.STEP_SLEEP)
        self.stepSpin.setMaximum(((abacus.MAX_SLEEP - abacus.MIN_SLEEP) // abacus.STEP_SLEEP) * abacus.STEP_SLEEP)
        self.stepSpin.setSingleStep(abacus.STEP_SLEEP)

        self.plot.setLabel('left', "Counts")
        self.plot.setLabel('bottom', "Sleep time", units='ns')

    def startStop(self):
        if self.startStopButton.text() == "Stop":
            self.timer.stop()
            self.completed = True
            self.updatePlot()
            self.completed = True

        else:
            step = self.stepSpin.value()
            n = self.nSpin.value()
            range_ = np.arange(self.startSpin.value(), self.stopSpin.value() + step, step)
            range_ = range_[range_ <= abacus.MAX_SLEEP]
            channel = self.comboBox.currentText()

            if self.parent.experiment != None:
                if self.parent.streaming:
                    if self.stopAcquisition():
                        self.run(channel, n, range_)
                else:
                    self.run(channel, n, range_)
            else:
                self.parent.connect()
                if self.parent.experiment != None:
                    if self.parent.streaming:
                        if self.stopAcquisition():
                            self.run(channel, n, range_)
                    else:
                        self.run(channel, n, range_)

    def run(self, channel, n, range_):
        self.cleanPlot()
        self.completed = False
        self.startStopButton.setText("Stop")
        self.startStopButton.setStyleSheet("background-color: green")
        self.enableWidgets(False)

        self.header = "Sleep time (ns)"  + constants.DELIMITER +  "Counts (%s)"%channel

        self.parent.check_timer.stop()
        thread = Thread(target = self.heavyDuty, args = (channel, n, range_))
        thread.daemon = True
        self.timer.start()
        thread.start()

    def heavyDuty(self, channel, n, range_):
        try:
            for (i, delay) in enumerate(range_):
                if not self.completed:
                    # result = self.parent.experiment.sleepSweep(channel, delay, n)
                    values = self.parent.experiment.sleepSweep(channel, delay, n)
                    result = []
                    for value in values:
                        result.append(value)
                    self.x_data.append(delay)
                    self.y_data.append(np.mean(result))
                else:
                    break

            self.completed = True
        except Exception as e:
            self.completed = True
            self.error = e



import os
import re
import sys
import webbrowser
import numpy as np
import abacusSoftware.__GUI_images__
import pyqtgraph as pg
from datetime import datetime
from time import time, localtime, strftime
from PyQt5 import QtCore, QtGui, QtWidgets

import abacusSoftware.constants as constants
import abacusSoftware.common as common
import abacusSoftware.builtin as builtin
import abacusSoftware.url as url
from abacusSoftware.menuBar import AboutWindow
from abacusSoftware.exceptions import ExtentionError
from abacusSoftware.files import ResultsFiles, RingBuffer
from abacusSoftware.supportWidgets import Table, CurrentLabels, ConnectDialog, SettingsDialog, SubWindow, ClickableLineEdit

import pyAbacus as abacus
from pyAbacus.communication import findPorts, CommunicationPort

# if constants.CURRENT_OS == "win32":
#     import win_unicode_console
#     win_unicode_console.enable()

common.readConstantsFile()

class MainWindow(QtWidgets.QMainWindow):
    count = 0

    def __init__(self, parent = None):
        super(QtWidgets.QMainWindow, self).__init__(parent)
        widget = QtWidgets.QWidget()

        layout = QtWidgets.QVBoxLayout(widget)

        layout.setContentsMargins(11, 11, 11, 11)
        layout.setSpacing(6)

        frame = QtWidgets.QFrame()
        frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        frame.setFrameShadow(QtWidgets.QFrame.Raised)

        horizontalLayout =  QtWidgets.QHBoxLayout(frame)
        label = QtWidgets.QLabel("Save as:")

        self.save_as_lineEdit = ClickableLineEdit()
        self.save_as_lineEdit.clicked.connect(self.chooseFile)

        self.save_as_button = QtWidgets.QPushButton("Open")

        horizontalLayout.addWidget(label)
        horizontalLayout.addWidget(self.save_as_lineEdit)
        horizontalLayout.addWidget(self.save_as_button)

        layout.addWidget(frame)

        frame2 = QtWidgets.QFrame()
        layout2 = QtWidgets.QHBoxLayout(frame2)
        layout.setSpacing(0)

        self.connect_button = QtWidgets.QPushButton("Connect")
        self.connect_button.setMaximumSize(QtCore.QSize(140, 60))
        layout2.addWidget(self.connect_button)
        self.acquisition_button = QtWidgets.QPushButton("Start Acquisition")
        self.acquisition_button.setMaximumSize(QtCore.QSize(140, 60))
        layout2.addWidget(self.acquisition_button)
        layout.addWidget(frame2)

        self.mdi = QtWidgets.QMdiArea(widget)
        self.mdi.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        layout.addWidget(self.mdi)
        self.setCentralWidget(widget)

        """
        settings
        """
        self.subPlots()
        self.subwindow_plots.show()

        self.subHistorical()
        self.subwindow_historical.show()

        self.subCurrent()
        self.subwindow_current.show()

        self.subSettings()
        self.subwindow_settings.show()

        """
        Config
        """
        self.setSettings()
        self.updateConstants()

        self.port = None
        self.port_name = None
        self.streaming = False
        self.experiment = None
        self.acquisition_button.clicked.connect(self.startAcquisition)

        self.connect_dialog = None
        self.connect_button.clicked.connect(self.connect)

        self.sampling_comboBox.currentIndexChanged.connect(self.samplingMethod)
        self.coincidence_spinBox.valueChanged.connect(self.coincidenceWindowMethod)
        self.delayA_spinBox.valueChanged.connect(self.delayAMethod)
        self.delayB_spinBox.valueChanged.connect(self.delayBMethod)
        self.sleepA_spinBox.valueChanged.connect(self.sleepAMethod)
        self.sleepB_spinBox.valueChanged.connect(self.sleepBMethod)

        self.initPlots()
        self.refresh_timer = QtCore.QTimer()
        self.refresh_timer.setInterval(constants.DATA_REFRESH_RATE)
        self.refresh_timer.timeout.connect(self.updateWidgets)

        self.data_timer = QtCore.QTimer()
        self.data_timer.setInterval(constants.DATA_REFRESH_RATE)
        self.data_timer.timeout.connect(self.updateData)

        self.check_timer = QtCore.QTimer()
        self.check_timer.setInterval(constants.CHECK_RATE)
        self.check_timer.timeout.connect(self.checkParams)

        self.results_files = None
        self.params_buffer = ""
        self.init_time = 0
        self.init_date = datetime.now().strftime('%Y/%m/%d %H:%M:%S')

        self.data_ring = RingBuffer(constants.BUFFER_ROWS, 4)

        self.save_as_button.clicked.connect(self.chooseFile)
        self.save_as_lineEdit.returnPressed.connect(self.setSaveAs)

        self.unlockSettings(True)

        """
        MenuBar
        """
        self.menubar = self.menuBar()

        self.menuFile = self.menubar.addMenu("File")
        self.menuProperties = self.menubar.addMenu("Properties")
        self.menuBuildIn = self.menubar.addMenu("Built In")
        self.menuView = self.menubar.addMenu("View")
        self.menuHelp = self.menubar.addMenu("Help")

        self.menuBuildInSweep = QtGui.QMenu("Sweep")

        sleepSweep = QtWidgets.QAction('Sleep time', self)
        delaySweep = QtWidgets.QAction('Delay time', self)

        self.menuBuildInSweep.addAction(sleepSweep)
        self.menuBuildInSweep.addAction(delaySweep)
        sleepSweep.triggered.connect(self.sleepSweep)
        delaySweep.triggered.connect(self.delaySweep)

        self.menuBuildIn.addMenu(self.menuBuildInSweep)

        self.statusBar = QtWidgets.QStatusBar(self)
        self.statusBar.setObjectName("statusBar")
        self.setStatusBar(self.statusBar)

        self.actionAbout = QtWidgets.QAction('About', self)
        self.actionSave_as = QtWidgets.QAction('Save as', self)
        self.actionDefault_settings = QtWidgets.QAction('Default settings', self)
        self.actionExit = QtWidgets.QAction('Exit', self)

        self.menuFile.addAction(self.actionSave_as)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionExit)
        self.menuHelp.addAction(self.actionAbout)
        self.menuProperties.addAction(self.actionDefault_settings)


        self.menuView.addAction(QtGui.QAction("Show settings", self.menubar, checkable=True))
        self.menuView.addAction(QtGui.QAction("Show historical", self.menubar, checkable=True))
        self.menuView.addAction(QtGui.QAction("Show current", self.menubar, checkable=True))
        self.menuView.addAction(QtGui.QAction("Show plots", self.menubar, checkable=True))
        self.menuView.addSeparator()
        self.menuView.addAction("Tiled")
        self.menuView.addAction("Cascade")

        for action in self.menuView.actions():
            if action.isCheckable(): action.setChecked(True)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuProperties.menuAction())
        self.menubar.addAction(self.menuBuildIn.menuAction())
        self.menubar.addAction(self.menuView.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.menuView.triggered[QtWidgets.QAction].connect(self.handleViews)

        self.actionSave_as.triggered.connect(self.chooseFile)
        self.actionSave_as.setShortcut("Ctrl+S")
        self.actionDefault_settings.triggered.connect(self.settingsDialogCaller)

        self.actionAbout.triggered.connect(self.aboutWindowCaller)
        self.actionExit.triggered.connect(self.close)
        self.actionExit.setShortcut("Ctrl+Q")

        self.acquisition_button.setDisabled(True)
        self.about_window = AboutWindow()
        self.settings_dialog = SettingsDialog(self)

        self.resize(800, 700)
        self.setWindowTitle(constants.WINDOW_NAME)
        self.showMaximized()

        # self.centerOnScreen()

        self.sleepSweepDialog = builtin.SleepDialog(self)
        self.delaySweepDialog = builtin.DelayDialog(self)

        self.mdi.tileSubWindows()
        # self.mdi.cascadeSubWindows()
        # self.subwindow_plots.resize(400, 350)

    def centerOnScreen(self):
        resolution = QtGui.QDesktopWidget().screenGeometry()
        sw = resolution.width()
        fw = self.frameSize().width()
        print(sw, fw)
        self.move((sw - fw) / 2,
                  (resolution.height() / 2) - (self.frameSize().height() / 2))

    def handleViews(self, q):
        text = q.text()
        if "Show" in text:
            for action in self.menuView.actions():
                if text == action.text():
                    text = text[5:]
                    check = not action.isChecked()
                    if check:
                        action.setChecked(False)
                        exec("self.subwindow_%s.hide()"%text)
                    else:
                        action.setChecked(True)
                        exec("self.subwindow_%s.show()"%text)

        elif text == "Cascade":
            self.mdi.cascadeSubWindows()

        elif text == "Tiled":
            self.mdi.tileSubWindows()

    def sleepSweep(self):
        self.sleepSweepDialog.show()

    def delaySweep(self):
        self.delaySweepDialog.show()

    def subSettings(self):
        def fillFormLayout(layout, values):
            for (i, line) in enumerate(values):
                layout.setWidget(i, QtWidgets.QFormLayout.LabelRole, line[0])
                layout.setWidget(i, QtWidgets.QFormLayout.FieldRole, line[1])

        settings_frame = QtWidgets.QFrame()
        settings_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        settings_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        settings_verticalLayout = QtWidgets.QVBoxLayout(settings_frame)
        settings_verticalLayout.setContentsMargins(11, 11, 11, 11)
        settings_verticalLayout.setSpacing(0)

        settings_frame2 = QtWidgets.QFrame()
        settings_frame2.setFrameShape(QtWidgets.QFrame.StyledPanel)
        settings_frame2.setFrameShadow(QtWidgets.QFrame.Raised)

        settings_frame2_formLayout =  QtWidgets.QFormLayout(settings_frame2)

        self.sampling_label = QtWidgets.QLabel("Sampling time:")
        self.sampling_comboBox = QtWidgets.QComboBox()
        self.coincidence_label = QtWidgets.QLabel("Coincidence window (ns):")
        self.coincidence_spinBox = QtWidgets.QSpinBox()
        self.delayA_label = QtWidgets.QLabel("Delay A (ns):")
        self.delayA_spinBox = QtWidgets.QSpinBox()
        self.delayB_label = QtWidgets.QLabel("Delay B (ns):")
        self.delayB_spinBox = QtWidgets.QSpinBox()
        self.sleepA_label = QtWidgets.QLabel("Sleep time A (ns):")
        self.sleepA_spinBox = QtWidgets.QSpinBox()
        self.sleepB_label = QtWidgets.QLabel("Sleep time B (ns):")
        self.sleepB_spinBox = QtWidgets.QSpinBox()

        self.sampling_comboBox.setEditable(True)
        self.sampling_comboBox.lineEdit().setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.sampling_comboBox.lineEdit().setReadOnly(True)

        self.coincidence_spinBox.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.delayA_spinBox.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.delayB_spinBox.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.sleepA_spinBox.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.sleepB_spinBox.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        widgets = [(self.sampling_label, self.sampling_comboBox),
                    (self.coincidence_label, self.coincidence_spinBox),
                    (self.delayA_label, self.delayA_spinBox),
                    (self.delayB_label, self.delayB_spinBox),
                    (self.sleepA_label, self.sleepA_spinBox),
                    (self.sleepB_label, self.sleepB_spinBox),]

        fillFormLayout(settings_frame2_formLayout, widgets)

        self.unlock_settings_button = QtWidgets.QPushButton("Unlock settings")
        self.unlock_settings_button.clicked.connect(lambda: self.unlockSettings(True))
        settings_frame2_formLayout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.unlock_settings_button)

        # settings_verticalLayout.addWidget(settings_frame3)
        settings_verticalLayout.addWidget(settings_frame2)

        # self.subwindow_settings = QtWidgets.QMdiSubWindow(self.mdi)
        self.subwindow_settings = SubWindow(self)
        self.subwindow_settings.setWidget(settings_frame)
        self.subwindow_settings.setWindowTitle("Settings")

        # self.subwindow_settings
        self.mdi.addSubWindow(self.subwindow_settings)

        self.setSettings()

    def unlockSettings(self, unlock = True):
        self.sampling_comboBox.setEnabled(unlock)
        self.coincidence_spinBox.setEnabled(unlock)
        self.delayA_spinBox.setEnabled(unlock)
        self.delayB_spinBox.setEnabled(unlock)
        self.sleepA_spinBox.setEnabled(unlock)
        self.sleepB_spinBox.setEnabled(unlock)
        if unlock:
            self.unlock_settings_button.setEnabled(False)
        else:
            self.unlock_settings_button.setEnabled(True)

    def subHistorical(self):
        widget = QtWidgets.QWidget()
        self.historical_table = Table(4)
        historical_layout = QtGui.QVBoxLayout(widget)
        historical_layout.addWidget(self.historical_table)

        self.subwindow_historical = SubWindow(self)
        self.subwindow_historical.setWidget(widget)
        self.subwindow_historical.setWindowTitle("Historical")
        self.mdi.addSubWindow(self.subwindow_historical)

    def subCurrent(self):
        widget = QtWidgets.QWidget()
        self.current_labels = CurrentLabels(widget)

        self.subwindow_current = SubWindow(self)
        self.subwindow_current.setMinimumSize(200, 100)
        self.subwindow_current.setWidget(widget)
        self.subwindow_current.setWindowTitle("Current")
        self.mdi.addSubWindow(self.subwindow_current)

    def subPlots(self):
        pg.setConfigOptions(foreground = 'k', background = None, antialias = True)
        self.plot_win = pg.GraphicsWindow()

        self.subwindow_plots = SubWindow(self)
        self.subwindow_plots.setWidget(self.plot_win)
        self.subwindow_plots.setWindowTitle("Plots")
        self.mdi.addSubWindow(self.subwindow_plots)

    def aboutWindowCaller(self):
        self.about_window.show()

    def settingsDialogCaller(self):
        self.settings_dialog.show()

    def timeInUnitsToMs(self, time):
        value = 0
        if 'ms' in time:
            value = int(time.replace('ms', ''))
        elif 's' in time:
            value = int(time.replace('s', ''))*1000
        return value

    def sendSettings(self):
        self.samplingMethod(self.sampling_comboBox.currentIndex())
        self.coincidenceWindowMethod(self.coincidence_spinBox.value())
        self.delayAMethod(self.delayA_spinBox.value())
        self.delayBMethod(self.delayB_spinBox.value())
        self.sleepAMethod(self.sleepA_spinBox.value())
        self.sleepBMethod(self.sleepB_spinBox.value())

    def samplingMethod(self, index):
        text_value = self.sampling_comboBox.currentText()
        value = common.timeInUnitsToMs(text_value)
        if value > 0 and self.experiment != None:
            if value > constants.DATA_REFRESH_RATE:
                self.refresh_timer.setInterval(value)
            else:
                self.refresh_timer.setInterval(constants.DATA_REFRESH_RATE)

            self.data_timer.setInterval(value)
            if self.results_files != None:
                self.results_files.writeParams("Sampling time (ms),%s"%value)
            try:
                self.experiment.setSampling(value)
            except abacus.exceptions.ExperimentError as e:
                self.errorWindow(e)

            self.sleepSweepDialog.samplingLabel.setText(text_value)
            self.delaySweepDialog.samplingLabel.setText(text_value)
        else:
            print("Sampling Value, %d"%value)

    def coincidenceWindowMethod(self, val):
        if self.experiment != None:
            try:
                self.experiment.setCoinWindow(val)
                if self.results_files != None:
                    self.results_files.writeParams("Coincidence Window (ns), %s"%str(val))
            except abacus.exceptions.ExperimentError as e:
                self.errorWindow(e)
        else:
            print("Coincidence Window Value: %d"%val)

    def delayAMethod(self, val):
        if self.experiment != None:
            try:
                self.experiment.detectors[0].setDelay(val)
                if self.results_files != None:
                    self.results_files.writeParams("Delay A (ns), %s"%str(val))
            except abacus.exceptions.ExperimentError as e:
                self.errorWindow(e)
        else:
            print("Delay A Value: %d"%val)

    def delayBMethod(self, val):
        if self.experiment != None:
            try:
                self.experiment.detectors[1].setDelay(val)
                if self.results_files != None:
                    self.results_files.writeParams("Delay B (ns), %s"%str(val))
            except abacus.exceptions.ExperimentError as e:
                self.errorWindow(e)
        else:
            print("Delay B Value: %d"%val)

    def sleepAMethod(self, val):
        if self.experiment != None:
            try:
                self.experiment.detectors[0].setSleep(val)
                if self.results_files != None:
                    self.results_files.writeParams("Sleep A (ns), %s"%str(val))
            except abacus.exceptions.ExperimentError as e:
                self.errorWindow(e)
        else:
            print("Sleep A Value: %d"%val)

    def sleepBMethod(self, val):
        if self.experiment != None:
            try:
                self.experiment.detectors[1].setSleep(val)
                if self.results_files != None:
                    self.results_files.writeParams("Sleep B (ns), %s"%str(val))
            except abacus.exceptions.ExperimentError as e:
                self.errorWindow(e)
        else:
            print("Sleep B Value: %d"%val)

    def checkParams(self):
        if self.experiment != None:
            try:
                self.experiment.periodicCheck()
                samp = self.experiment.getSamplingValue()
                coin = self.experiment.getCoinwinValue()
                values = self.experiment.getDetectorsTimersValues()
                (dA, sA), (dB, sB) = values

                if common.timeInUnitsToMs(self.sampling_comboBox.currentText()) != samp:
                    if samp >= 1000:
                        index = self.sampling_comboBox.findText('%d s'%(samp/1000))
                    else:
                        index = self.sampling_comboBox.findText('%d ms'%samp)
                    self.sampling_comboBox.setCurrentIndex(index)
                if self.coincidence_spinBox.value() != coin:
                    self.coincidence_spinBox.setValue(coin)
                if self.delayA_spinBox.value() != dA:
                    self.delayA_spinBox.setValue(dA)
                if self.delayB_spinBox.value() != dB:
                    self.delayB_spinBox.setValue(dB)
                if self.sleepA_spinBox.value() != sA:
                    self.sleepA_spinBox.setValue(sA)
                if self.sleepB_spinBox.value() != sB:
                    self.sleepB_spinBox.setValue(sB)
            except abacus.exceptions.ExperimentError as e:
                self.errorWindow(e)

    def unlockSettings(self, unlock = True):
        self.sampling_comboBox.setEnabled(unlock)
        self.coincidence_spinBox.setEnabled(unlock)
        self.delayA_spinBox.setEnabled(unlock)
        self.delayB_spinBox.setEnabled(unlock)
        self.sleepA_spinBox.setEnabled(unlock)
        self.sleepB_spinBox.setEnabled(unlock)
        if unlock:
            self.unlock_settings_button.setEnabled(False)
        else:
            self.unlock_settings_button.setEnabled(True)

    def setSettings(self):
        common.setSamplingComboBox(self.sampling_comboBox)
        common.setCoincidenceSpinBox(self.coincidence_spinBox)
        common.setDelaySpinBox(self.delayA_spinBox)
        common.setDelaySpinBox(self.delayB_spinBox)
        common.setSleepSpinBox(self.sleepA_spinBox)
        common.setSleepSpinBox(self.sleepB_spinBox)

    def cleanPort(self):
        if self.streaming:
            self.startAcquisition()

        if self.port != None:
            self.port.close()
            self.port_name = None
            self.port = None

        if self.experiment != None:
            self.check_timer.stop()
            self.experiment = None

    def connect(self):
        if self.port != None:
            self.connect_button.setText("Connect")
            self.acquisition_button.setDisabled(True)
            if self.results_files != None:
                self.results_files.writeParams("Disconnected from device in port,%s"%self.port_name)
            self.cleanPort()
        else:
            self.connect_dialog = ConnectDialog()
            self.connect_dialog.refresh()
            self.connect_dialog.exec_()

            port = self.connect_dialog.comboBox.currentText()

            if port != "":
                self.port_name = port
                self.port = CommunicationPort(self.connect_dialog.ports[self.port_name])
                self.experiment = abacus.Experiment(self.port)

                self.acquisition_button.setDisabled(False)
                self.acquisition_button.setStyleSheet("background-color: none")
                self.acquisition_button.setText("Start acquisition")
                self.connect_button.setText("Disconnect")

                if len(self.current_labels.labels) == 0:
                    self.current_labels.createLabels()
                    self.current_labels.setColors(["red", "blue", "black"])
                # self.checkParams()
                self.check_timer.start()

                msg = "Connected to device in port,%s"%self.port_name
                if self.results_files != None:
                    try:
                        self.results_files.writeParams(msg)
                    except FileNotFoundError as e:
                        self.errorWindow(e)
                else:
                    self.params_buffer += constants.BREAKLINE + strftime("%H:%M:%S", localtime()) + "," + msg

            else:
                self.connect_button.setText("Connect")
                self.acquisition_button.setDisabled(True)

    def startClocks(self):
        self.refresh_timer.start()
        self.data_timer.start()

    def stopClocks(self):
        self.refresh_timer.stop()
        self.data_timer.stop()
        try:
            self.data_ring.save()
        except FileNotFoundError as e:
            self.errorWindow(e)

    def startAcquisition(self):
        if self.port == None:
            QtWidgets.QMessageBox.warning(self, 'Error', "Port has not been choosed", QtWidgets.QMessageBox.Ok)
        elif self.results_files != None:
            if self.streaming:
                self.acquisition_button.setStyleSheet("background-color: none")
                self.acquisition_button.setText("Start acquisition")
                self.results_files.writeParams("Acquisition stopped")
                self.unlockSettings()
                self.stopClocks()
            else:
                self.acquisition_button.setStyleSheet("background-color: green")
                self.acquisition_button.setText("Stop acquisition")
                self.results_files.writeParams("Acquisition started")
                self.sendSettings()
                self.unlockSettings(False)
                self.startClocks()

            self.streaming = not self.streaming
            if self.init_time == 0:
                self.init_time = time()
        else:
            QtWidgets.QMessageBox.warning(self, 'Error', "Please choose an output file.", QtWidgets.QMessageBox.Ok)

    def updateData(self):
        try:
            time_, detectors, coins = self.experiment.currentValues()
            time_ += -self.init_time

            values = np.array([time_] + detectors + coins)
            values = values.reshape((1, values.shape[0]))
            self.data_ring.extend(values)
        except abacus.exceptions.ExperimentError as e:
            self.errorWindow(e)
        except FileNotFoundError as e:
            self.errorWindow(e)

    def updateWidgets(self):
        data = self.data_ring[:]

        self.updatePlots(data)
        self.updateTable(data)
        self.updateCurrents(data)

    def updateTable(self, data):
        self.historical_table.insertData(data)

    def updateCurrents(self, data):
        for i in range(1, 4):
            self.current_labels.changeValue(i-1, data[-1, i])

    def updatePlots(self, data):
        time_ = data[:, 0]

        self.countsA_line.setData(time_, data[:, 1])
        self.countsB_line.setData(time_, data[:, 2])

        self.coins_line.setData(time_, data[:, 3])

    def initPlots(self):
        self.counts_plot = self.plot_win.addPlot()
        self.coins_plot = self.plot_win.addPlot(row = 1, col = 0)

        self.counts_plot.addLegend()
        self.coins_plot.addLegend()

        symbolSize = 5
        self.countsA_line = self.counts_plot.plot(pen = "r", symbol='o', symbolPen = "r", symbolBrush="r", symbolSize=symbolSize, name="Detector A")
        self.countsB_line = self.counts_plot.plot(pen = "b", symbol='o', symbolPen = "b", symbolBrush="b", symbolSize=symbolSize, name="Detector B")

        self.coins_line = self.coins_plot.plot(pen = "k", symbol='o', symbolPen = "k", symbolBrush="k", symbolSize=symbolSize, name="Coincidences AB")

        self.counts_plot.setLabel('left', "Counts")
        self.coins_plot.setLabel('left', "Coincidences")
        self.coins_plot.setLabel('bottom', "Time", units='s')

    def setSaveAs(self):
        new_file_name = self.save_as_lineEdit.text()
        try:
            if new_file_name != "":
                try:
                    name, ext = self.checkFileName(new_file_name)
                    if self.results_files == None:
                        self.results_files = ResultsFiles(name, ext, self.init_date)
                        self.results_files.params_file.header += self.params_buffer
                        self.params_buffer = ""
                    else:
                        self.results_files.changeName(name, ext)
                    names = self.results_files.getNames()
                    self.data_ring.setFile(self.results_files.data_file)
                    self.statusBar.showMessage('Files: %s, %s.'%(names))
                    try:
                        self.results_files.checkFilesExists()
                    except FileExistsError:
                        print("FileExistsError")
                except ExtentionError as e:
                    self.save_as_lineEdit.setText("")
                    self.errorWindow(e)
            else:
                print("EmptyName")
        except FileNotFoundError as e:
            self.errorWindow(e)

    def checkFileName(self, name):
        if "." in name:
            name, ext = name.split(".")
            ext = ".%s"%ext
        else:
            try:
                ext = constants.extension_comboBox
                print(ext)
            except AttributeError:
                ext = constants.EXTENSION_DATA
            name = common.unicodePath(name)
            self.save_as_lineEdit.setText(name + ext)
        if ext in constants.SUPPORTED_EXTENSIONS.keys():
            return name, ext
        else:
            raise ExtentionError()

    def chooseFile(self):
        """
        user interaction with saving file
        """
        try:
            directory = constants.directory_lineEdit
        except:
            directory = os.path.expanduser("~")

        dlg = QtWidgets.QFileDialog(directory = directory)
        dlg.setAcceptMode(QtWidgets.QFileDialog.AcceptSave)
        dlg.setFileMode(QtWidgets.QFileDialog.AnyFile)
        nameFilters = [constants.SUPPORTED_EXTENSIONS[extension] for extension in constants.SUPPORTED_EXTENSIONS]
        dlg.setNameFilters(nameFilters)
        dlg.selectNameFilter(constants.SUPPORTED_EXTENSIONS[constants.EXTENSION_DATA])
        if dlg.exec_():
            name = dlg.selectedFiles()[0]
            self.save_as_lineEdit.setText(common.unicodePath(name))
            self.setSaveAs()

    def updateConstants(self):
        try:
            common.updateConstants(self)

            if constants.autogenerate_checkBox:
                file_name = constants.file_prefix_lineEdit
                if constants.datetime_checkBox: file_name += strftime("_%Y%m%d_%H%M")
                file_name += constants.extension_comboBox
                path = os.path.join(constants.directory_lineEdit, file_name)
                self.save_as_lineEdit.setText(common.unicodePath(path))
                self.setSaveAs()

            self.data_ring.updateFormat(delimiter = constants.DELIMITER)

        except AttributeError:
            pass

    def errorWindow(self, exception):
        error_text = str(exception)
        msg = QtWidgets.QMessageBox()
        msg.setIcon(QtWidgets.QMessageBox.Warning)

        if type(exception) is abacus.exceptions.ExperimentError:
            self.stopClocks()
            self.cleanPort()
            self.experiment = None
            self.streaming = False
            self.acquisition_button.setDisabled(True)
            self.acquisition_button.setStyleSheet("background-color: red")
            msg.setIcon(QtWidgets.QMessageBox.Critical)
            self.connect_button.setText("Connect")

        try:
            self.results_files.writeParams("Error,%s"%error_text)
        except Exception:
            pass

        msg.setText('An Error has ocurred.\n%s'%error_text)
        msg.setWindowTitle("Error")
        msg.setStandardButtons(QtWidgets.QMessageBox.Ok | QtWidgets.QMessageBox.Cancel)
        msg.exec_()

    def closeEvent(self, event):
        if self.results_files == None:
            event.accept()
        elif self.results_files.areEmpty():
            event.accept()
        elif self.results_files.data_file.isEmpty():
            self.results_files.params_file.delete()
            event.accept()
        else:
            quit_msg = "Are you sure you want to exit the program?"
            reply = QtWidgets.QMessageBox.question(self, 'Exit',
                             quit_msg, QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
            if reply == QtWidgets.QMessageBox.Yes:
                event.accept()
            else:
                event.ignore()

    def initial(self):
        self.__sleep_timer__.stop()
        # self.softwareUpdate()
        self.connect()

    def show2(self):
        self.show()

        self.__sleep_timer__ = QtCore.QTimer()
        self.__sleep_timer__.setInterval(10)
        self.__sleep_timer__.timeout.connect(self.initial)
        self.__sleep_timer__.start()

def softwareUpdate(splash):
    try:
        check = constants.check_updates_checkBox
    except:
        if constants.SETTING_FILE_EXISTS:
            os.remove(constants.SETTINGS_PATH)
        check = True
    if check:
        version = url.checkUpdate()
        if version != None:
            splash.close()
            msg = QtWidgets.QMessageBox()
            msg.setIcon(QtWidgets.QMessageBox.Information)
            msg.setText("There is a new version avaible (%s).\nDo you want to download it?"%version)
            msg.setWindowTitle("Update avaible")
            msg.setStandardButtons(QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
            if msg.exec_() == QtWidgets.QMessageBox.Yes:
                webbrowser.open(url.TARGET_URL)
                exit()

def run():
    from time import sleep

    app = QtWidgets.QApplication(sys.argv)
    QtWidgets.QApplication.setStyle(QtWidgets.QStyleFactory.create('Fusion')) # <- Choose the style

    splash_pix = QtGui.QPixmap(':/splash.png').scaledToWidth(600)
    splash = QtWidgets.QSplashScreen(splash_pix, QtCore.Qt.WindowStaysOnTopHint)
    splash.show()

    icon = QtGui.QIcon(':/abacus_small.ico')
    app.setWindowIcon(icon)
    app.processEvents()

    if abacus.CURRENT_OS == 'win32':
        import ctypes
        myappid = 'abacus.abacus.01' # arbitrary string
        ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)

    sleep(1)

    softwareUpdate(splash)
    splash.close()

    main = MainWindow()
    main.setWindowIcon(icon)

    main.show2()
    app.exec_()

if __name__ == "__main__":
    run()
    sys.exit()



import os
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "abacusSoftware",
    version = "1.2.0",
    author = "Juan Barbosa",
    author_email = "js.barbosa10@uniandes.edu.co",
    description = ("Abacus Software is a suite of tools build to ensure your experience with Tausand's coincidence counters becomes simplified."),
    license = "GPL",
    keywords = "example documentation tutorial",
    url = "https://github.com/Tausand-dev/ReimaginedQuantum",
    packages=['abacusSoftware'],
    install_requires = ['pyAbacus', 'pyserial', 'numpy', 'pyqtgraph', 'PyQt5'],
    long_description = "",#read('README'),
    classifiers = [
        "Development Status :: 1 - Planning",
        "Topic :: Utilities",
        "License :: OSI Approved :: GNU General Public License (GPL)",
    ],
)
